@@ -515,20 +515,23 @@ ClientList Workspace::constrainedStackingOrder()
         kDebug(1212) << (void*)(*it) << *it << ":" << (*it)->layer();
 #endif
     // build the order from layers
-    QHash< Group*, Layer > minimum_layer;
-    for (ClientList::ConstIterator it = unconstrained_stacking_order.constBegin();
-            it != unconstrained_stacking_order.constEnd();
-            ++it) {
+    QVector< QMap<Group*, Layer> > minimum_layer(numScreens());
+    for (ClientList::ConstIterator it = unconstrained_stacking_order.constBegin(),
+                                  end = unconstrained_stacking_order.constEnd(); it != end; ++it) {
         Layer l = (*it)->layer();
-        // If a window is raised above some other window in the same window group
-        // which is in the ActiveLayer (i.e. it's fulscreened), make sure it stays
-        // above that window (see #95731).
-        if (minimum_layer.contains((*it)->group())
-                && minimum_layer[(*it)->group()] == ActiveLayer
-                && (l == NormalLayer || l == AboveLayer)) {
-            l = minimum_layer[(*it)->group()];
+
+        const int screen = (*it)->screen();
+        QMap< Group*, Layer >::iterator mLayer = minimum_layer[screen].find((*it)->group());
+        if (mLayer != minimum_layer[screen].end()) {
+            // If a window is raised above some other window in the same window group
+            // which is in the ActiveLayer (i.e. it's fulscreened), make sure it stays
+            // above that window (see #95731).
+            if (*mLayer == ActiveLayer && (l == NormalLayer || l == AboveLayer))
+                l = ActiveLayer;
+            *mLayer = l;
+        } else {
+            minimum_layer[screen].insertMulti((*it)->group(), l);
         }
-        minimum_layer[(*it)->group()] = l;
         layer[ l ].append(*it);
     }
     ClientList stacking;
@@ -838,15 +841,37 @@ Layer Client::belongsToLayer() const
     return NormalLayer;
 }
 
+bool rec_checkTransientOnTop(const ClientList &transients, const Client *topmost)
+{
+    foreach (const Client *transient, transients) {
+        if (transient == topmost || rec_checkTransientOnTop(transient->transients(), topmost)) {
+            return true;
+        }
+    }
+    return false;
+}
+
 bool Client::isActiveFullScreen() const
 {
+    if (!isFullScreen())
+        return false;
+
+//     const Client* ac = workspace()->mostRecentlyActivatedClient(); // instead of activeClient() - avoids flicker
+//     if (!ac)
+//         return false;
+// not needed, for xinerama  -> && ( ac == this || this->group() == ac->group())
+
     // only raise fullscreen above docks if it's the topmost window in unconstrained stacking order,
     // i.e. the window set to be topmost by the user (also includes transients of the fullscreen window)
-    const Client* ac = workspace()->mostRecentlyActivatedClient(); // instead of activeClient() - avoids flicker
     const Client* top = workspace()->topClientOnDesktop(workspace()->currentDesktop(), screen(), true, false);
-    return(isFullScreen() && ac != NULL && top != NULL
-// not needed, for xinerama  && ( ac == this || this->group() == ac->group())
-           && (top == this || this->group() == top->group()));
+    if (!top)
+        return false;
+
+    // check whether we ...
+    if (top == this)
+        return true;
+    // ... or one of our transients is topmost
+    return rec_checkTransientOnTop(transients_list, top);
 }
 
 } // namespace
